# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Denis Kovalskiy denimnumber1@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from quickly_ide_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('quickly-ide')

class ProjectadderDialog(Gtk.Dialog):
    __gtype_name__ = "ProjectadderDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated ProjectadderDialog object.
        """
        builder = get_builder('ProjectadderDialog')
        new_object = builder.get_object('projectadder_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a ProjectadderDialog object with it in order to
        finish initializing the start of the new ProjectadderDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)

    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        pass

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        pass
        
    def set_project_template(self, template):
        self.ui.project_template_entry.set_text(template)
        
    def get_project_name(self):
        return self.ui.project_name_entry.get_text()
        
    def get_project_path(self):
        return self.ui.project_path_bt.get_filename()            


if __name__ == "__main__":
    dialog = ProjectadderDialog()
    dialog.show()
    Gtk.main()
