# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Denis Kovalskiy denimnumber1@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('quickly-ide')

from gi.repository import Gtk, GLib, GdkPixbuf, Vte, WebKit, GtkSource # pylint: disable=E0611
import logging
logger = logging.getLogger('quickly_ide')

from quickly_ide_lib import Window
from quickly_ide_lib.helpers import *
from quickly_ide.AboutQuicklyIdeDialog import AboutQuicklyIdeDialog
from quickly_ide.PreferencesQuicklyIdeDialog import PreferencesQuicklyIdeDialog
from quickly_ide.ProjectadderDialog import ProjectadderDialog
from quickly_ide.TextDialog import TextDialog

import ConfigParser
import pickle

try:
    from quickly import api as quicklyapi
    import quickly.configurationhandler as quicklyconf
except:
    print "Quickly not found"
    Gtk.main_quit()    


COL_DESC = 0
COL_PIXBUF = 1
COL_CALLBACK = 2

START_PAGE = "http://developer.ubuntu.com/"

CONFIG_PATH = str(GLib.get_user_config_dir() + "/quickly-ide/")

command_order = ('create', 'run', 'edit', 'design', 'save', 'package', 'share', 'release', 'debug', 'tutorial')
command_without_output = ('edit', 'design', 'debug', 'tutorial')
command_add_types = ('dialog', 'indicator', 'help-guide', 'help-topic')

supported_templates = ('ubuntu-application')

# See quickly_ide_lib.Window.py for more details about how this class works
class QuicklyIdeWindow(Window):
    __gtype_name__ = "QuicklyIdeWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(QuicklyIdeWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutQuicklyIdeDialog
        self.PreferencesDialog = PreferencesQuicklyIdeDialog

        # Code for other initialization actions should be added here.
        self.restore_state()
        
        # Set toolbar color black
        context = self.ui.toolbar.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        
        # Set hidden tabs
        self.ui.notebook.set_show_tabs(False)
        
        self.ui.templates_view.set_text_column(COL_DESC)
        self.ui.templates_view.set_pixbuf_column(COL_PIXBUF)
        
        # Quickly templates
        self.templates = [
           {'desc': _('Ubuntu application'),
               'icon': GdkPixbuf.Pixbuf.new_from_file(get_media_file(
                            'background.png').replace("file://","")),
               'callback': 'create_ubuntu_app'},
        ]
        self.templates_store = Gtk.ListStore(str, GdkPixbuf.Pixbuf, str)
        # ^ desc, icon, callback ^
        self.templates_store.set_sort_column_id(COL_DESC,
                                               Gtk.SortType.ASCENDING)
        self.ui.templates_view.set_model(self.templates_store) 
        self.fill_templates_store()  
        
        # Setup terminal
        self.terminal = Vte.Terminal()
        self.ui.scrolledwindow3.add(self.terminal)
        
        # Setup devhelp view
        self.devhelp_view = WebKit.WebView()
        self.ui.devhelp_scrolledwindow.add(self.devhelp_view)
        self.websettings = WebKit.WebSettings()
        self.websettings.props.enable_accelerated_compositing = True
        self.websettings.props.enable_dns_prefetching = True
        self.websettings.props.enable_fullscreen = True
        self.websettings.props.enable_offline_web_application_cache = True
        self.websettings.props.enable_html5_database = True
        self.websettings.props.enable_html5_local_storage = True
        #self.websettings.props.enable_xss_auditor = True
        #self.websettings.props.enable_hyperlink_auditing = False
        #self.websettings.props.enable_file_access_from_file_uris = True
        #self.websettings.props.enable_universal_access_from_file_uris = True
        self.websettings.props.enable_site_specific_quirks = True
        self.websettings.props.enable_spell_checking = True
        self.websettings.props.enable_webaudio = True
        self.websettings.props.enable_webgl = True
        self.websettings.props.enable_page_cache = True
        self.websettings.props.enable_plugins = True
        self.devhelp_view.set_settings(self.websettings)
        self.devhelp_view.open(START_PAGE)
        
        # Setup editor
        self.editorBuffer = GtkSource.Buffer()
        self.editLangMan = GtkSource.LanguageManager()
        self.editor_current_file = ""
        self.editor = GtkSource.View()
        self.editor.set_buffer(self.editorBuffer)
        self.editor.set_show_line_numbers(True)
        self.editor.set_highlight_current_line(True)
        self.ui.scrolledwindow4.add(self.editor)
        
        # Setup project file view
        self.project_view_model = Gtk.ListStore(GdkPixbuf.Pixbuf, str)
        self.ui.editor_project_view.set_model(self.project_view_model)
        col = Gtk.TreeViewColumn()
        col.set_title(_('Project'))
        render_pixbuf = Gtk.CellRendererPixbuf()
        col.pack_start(render_pixbuf, expand=False)
        col.add_attribute(render_pixbuf, 'pixbuf', 0)
        render_text = Gtk.CellRendererText()
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', 1)
        col.set_sort_column_id(1)
        col.set_clickable(False)
        self.ui.editor_project_view.append_column(col)
        
        # Setup add tabs
        addTypeModel = Gtk.ListStore(str)
        for item in command_add_types:
            addTypeModel.append([item])
        self.ui.add_type_bt.set_model(addTypeModel)
        cell = Gtk.CellRendererText()
        self.ui.add_type_bt.pack_start(cell, True)
        self.ui.add_type_bt.add_attribute(cell, 'text', 0)
        self.ui.add_type_bt.set_active(0)
        
        # Setup project variables
        
        self.project_current_path = None
        self.project_root_dir = None
        self.project_template = None
        self.project_name = None
        
        self.set_project_bts_enabled(False)
        
        # Setup key shortcuts
        self.app_accelerators = Gtk.AccelGroup()
        self.add_accel_group(self.app_accelerators)
        self.add_accelerator(self.ui.edit_save_bt, "<Control>s", signal="clicked")
        
        # Setup project tabs
        self.projects_model = Gtk.ListStore(str, str, str)
        self.ui.projects_view.set_model(self.projects_model)
        col = Gtk.TreeViewColumn()
        col.set_title(_('Name'))
        render_text = Gtk.CellRendererText()
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', 0)
        col.set_sort_column_id(0)
        col.set_clickable(False)
        self.ui.projects_view.append_column(col)
        col = Gtk.TreeViewColumn()
        col.set_title(_('Template'))
        render_text = Gtk.CellRendererText()
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', 1)
        col.set_clickable(False)
        self.ui.projects_view.append_column(col)
        col = Gtk.TreeViewColumn()
        col.set_title(_('Path'))
        render_text = Gtk.CellRendererText()
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', 2)
        col.set_clickable(False)
        self.ui.projects_view.append_column(col)
        self.fill_projects_view()
    
    def on_mnu_new_activate(self, widget):
        self.create_ubuntu_app()
    
    def on_mnu_open_activate(self, widget):
        dialog = Gtk.FileChooserDialog(title=_("Open project folder"),action=Gtk.FileChooserAction.SELECT_FOLDER,
                        buttons=(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OPEN,Gtk.ResponseType.ACCEPT))

        response = dialog.run()
        if response == Gtk.ResponseType.ACCEPT:
            self.load_project(dialog.get_filename())
        elif response == Gtk.ResponseType.CANCEL:
            pass

        dialog.destroy()
    
    def on_mnu_save_as_activate(self, widget):
        self.on_edit_save_as_bt_clicked(widget)
    
    def on_mnu_delete_activate(self, widget):
        if self.ui.notebook.get_current_page() == 1:
            self.on_projects_remove_bt_clicked(widget)
        
    def project_load_conf(self):
        self.on_config_clear_bt_clicked(self)   
    
        if self.project_get_conf("bzrbranch") != None:
            self.ui.config_bzr_entry.set_text(self.project_get_conf("bzrbranch"))
            
        if self.project_get_conf("lp_id") != None:
            self.ui.config_lp_entry.set_text(self.project_get_conf("lp_id")) 
            
        if self.project_get_conf("ppa") != None:
            self.ui.config_ppa_entry.set_text(self.project_get_conf("ppa"))
            
        if self.project_get_conf("version") != None:
            self.ui.config_target_entry.set_text(self.project_get_conf("version"))                                   
            
        
        
    def load_project(self, path):
        if quicklyapi.get_root_project_path(path) == None:
            self.error_message(_("Error"), _("This is not quickly project"))
            return
            
        if quicklyapi.get_current_template(path) not in supported_templates:
            self.error_message(_("Error"), _("This template does not support yet"))
            return
            
        self.project_template = quicklyapi.get_current_template(path)
        self.project_root_dir = path
        self.project_current_path = path
        self.project_name = self.project_get_conf('project')
        self.project_load_conf()
        
        self.projects_add_project(self.project_name, self.project_template, self.project_root_dir)
        self.load_project_dir(self.project_root_dir)
        self.set_project_bts_enabled(True) 
        
        self.ui.info_name_entry.set_text(self.project_name)
        self.ui.info_type_entry.set_text(self.project_template)
        
        self.ui.statusLabel.set_text(self.project_name + " " + _("loaded"))        
    
    def reload_project(self):
        self.load_project(self.project_root_dir)
    
    def project_run_command(self, command, args = None, path = None):
        if command in command_without_output:
            quicklyapi.run_command(command, self.project_template, self.project_root_dir, "")
        else:
            Qcommand = []
            Qcommand.append("quickly")
            Qcommand.append(command)
            if path == None: path = self.project_root_dir
            if args != None:
                for arg in args:
                    Qcommand.append(arg)
            self.terminal.fork_command_full(Vte.PtyFlags.DEFAULT, path, Qcommand,
                            None, GLib.SpawnFlags.SEARCH_PATH, None, None)    
    
    def project_get_conf(self, key):
        quicklyconf.loadConfig(can_stop=False, config_file_path=self.project_root_dir)
        try:
            return quicklyconf.project_config[key]
        except KeyError:
            return None

    def add_accelerator(self, widget, accelerator, signal="activate"):
        """Adds a keyboard shortcut"""
        if accelerator is not None:
            #if DEBUG:
                #print accelerator, widget.get_tooltip_text()
            key, mod = Gtk.accelerator_parse(accelerator)
            widget.add_accelerator(signal, self.app_accelerators, key, mod, Gtk.AccelFlags.VISIBLE)
    
    def error_message(self, title, text):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,
            Gtk.ButtonsType.OK, title)
        dialog.format_secondary_text(text)
        dialog.run()

        dialog.destroy()
     
    def refresh_project_view(self): 
        self.load_project_dir(self.project_current_path) 
                                               
    def load_project_dir(self, path):
        dirList=os.listdir(path)
        dircontents = []
        filecontents = []
        for item in dirList: 
            if item[0] != '.' and item[0].startswith(".") == False:
                if os.path.isdir(path+"/"+item):
                    dircontents.append([item])
                else:
                    filecontents.append([item])
        dircontents.sort() 
        self.project_current_path = path
        self.project_view_model.clear()
        if (self.project_current_path != self.project_root_dir):
            self.project_view_model.append([Gtk.IconTheme.load_icon(Gtk.IconTheme.get_default(),"folder",16,0), ".."])
        folderIcon = Gtk.IconTheme.load_icon(Gtk.IconTheme.get_default(),"folder",16,0)
        fileIcon = Gtk.IconTheme.load_icon(Gtk.IconTheme.get_default(),"text-x-generic",16,0)
        uiIcon = Gtk.IconTheme.load_icon(Gtk.IconTheme.get_default(),"text-x-generic-template",16,0)
        imageIcon = Gtk.IconTheme.load_icon(Gtk.IconTheme.get_default(),"image-x-generic",16,0)    
        for act in dircontents:
            self.project_view_model.append([folderIcon, act[0]])
        for act in filecontents:
            if self.is_file_type(act[0], "ui"):
                self.project_view_model.append([uiIcon, act[0]])
            elif self.is_file_type(act[0], "image"):
                self.project_view_model.append([imageIcon, act[0]])
            elif self.is_file_type(act[0], "trash"):
                pass
            elif self.is_file_type(act[0], "pytrash"):
                pass           
            else:        
                self.project_view_model.append([fileIcon, act[0]])    
    
    def on_projects_view_row_activated(self, widget, row, col):
        model = widget.get_model()
        path = model[row][2]
        self.load_project(path)
            
    def on_editor_project_view_row_activated(self, widget, row, col):
        model = widget.get_model()
        target = model[row][1]
        path = str(self.project_current_path+"/"+target)
        if target == "..":
            self.load_project_dir(os.path.abspath(os.path.join(str(self.project_current_path), os.path.pardir)))
        elif os.path.isdir(path):
            self.load_project_dir(path)    
        elif os.path.isfile(path):
            if self.is_file_type(path, "image"):
                show_uri(self, "file://"+path)
            else:    
                self.editor_load_file(path) 
     
    def on_projects_add_bt_clicked(self, widget):
        self.on_mnu_open_activate(widget)
        
    def on_projects_remove_bt_clicked(self, widget):
        (model, titer) = self.ui.projects_view.get_selection().get_selected()
        index = int(str(model.get_path(titer)))
        del self.projects_list[index]
        self.fill_projects_view()
        
    def on_projects_refresh_bt_clicked(self, widget):
        self.fill_projects_view()          
                     
    def on_config_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(6)                                
    
    def projects_has_project(self, path):
        for project in self.projects_list:
            if path == project[2]:
                return True
    
    def projects_add_project(self, name, template, path):
        if self.projects_has_project(path) != True:
            self.projects_list.append([name, template, path])
            self.fill_projects_view()
    
    def fill_projects_view(self):
        self.projects_model.clear()

        for project in self.projects_list:
            if os.path.exists(project[2]):
                self.projects_model.append([project[0],
                                           project[1],
                                           project[2]])
        
    def fill_templates_store(self):
        self.templates_store.clear()
        
        for template in self.templates:
            self.templates_store.append([template['desc'],
                                        template['icon'],
                                        template['callback']])    
    
    def on_templates_view_item_activated(self, widget, item):
        model = widget.get_model()
        t_callback = getattr(QuicklyIdeWindow, model[item][COL_CALLBACK])

        t_callback(self)
    
    def create_ubuntu_app(self):
        dialog = ProjectadderDialog()
        dialog.set_project_template("ubuntu-application")
        response = dialog.run()
        if response == Gtk.ResponseType.OK: 
            name = dialog.get_project_name()
            path = dialog.get_project_path()
            if not os.access(path, os.W_OK): 
                self.error_message(_("Error"),_("Selected dir is not writable"))
            else:
                args = ["ubuntu-application", name]
                self.project_run_command("create", args, path)
                projectPath = str(path + "/" + name)
                self.projects_add_project(name, "ubuntu-application", projectPath)
                self.ui.notebook.set_current_page(2)                
        else:
            pass    

        dialog.destroy()   
        
    def on_create_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(0)
        
    def on_projects_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(1) 
        
    def on_active_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(2)
        
    def on_devhelp_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(3)
        
    def on_devhelp_refresh_bt_clicked(self, widget):
        self.devhelp_view.reload()
        
    def on_devhelp_goback_bt_clicked(self, widget):
        self.devhelp_view.go_back()
        
    def on_devhelp_gonext_bt_clicked(self, widget):
        self.devhelp_view.go_forward()
        
    def on_devhelp_zoom_in_bt_clicked(self, widget):
        self.devhelp_view.zoom_in()
    
    def on_devhelp_zoom_out_bt_clicked(self, widget):
        self.devhelp_view.zoom_out()                
        
    def on_add_clear_bt_clicked(self, widget):
        self.ui.add_name_entry.set_text("")    
        
    def on_fullscreen_bt_toggled(self, widget):
        if(self.ui.fullscreen_bt.get_active()):
            self.fullscreen()
        else:
            self.unfullscreen()
     
    def on_project_run_bt_clicked(self, widget):
        self.on_editor_run_bt_clicked(widget)
             
    def on_project_edit_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(4)    
    
    def on_project_design_bt_clicked(self, widget):
        self.on_editor_design_bt_clicked(widget)
    
    def on_project_package_bt_clicked(self, widget):
        self.project_run_command("package")
        
    def on_project_save_bt_clicked(self, widget):
        dialog = TextDialog()
        dialog.set_title(_("Get commit message"))
        
        response = dialog.run()
        
        if response == Gtk.ResponseType.OK:
            msg =  dialog.get_text()
            if msg != "":
                arg = [msg]
                self.project_run_command("save", arg)
        elif response == Gtk.ResponseType.CANCEL:
            pass    
        
        dialog.destroy()
            
        
    def on_project_share_bt_clicked(self, widget):
        self.project_run_command("share")
        
    def on_project_release_bt_clicked(self, widget):
        self.project_run_command("release") 
        
    def on_project_license_bt_clicked(self, widget):
        self.project_run_command("license")           
        
    def on_config_clear_bt_clicked(self, widget):
        self.ui.config_bzr_entry.set_text("")
        self.ui.config_lp_entry.set_text("")
        self.ui.config_ppa_entry.set_text("")
        self.ui.config_target_entry.set_text("")
        
    def on_config_rev_bt_clicked(self, widget):
        self.project_load_conf()
        
    def on_config_ok_bt_clicked(self, widget):
        bzr = self.project_get_conf("bzrbranch")
        lp = self.project_get_conf("lp_id")
        ppa = self.project_get_conf("ppa")
        target = self.project_get_conf("version")
        
        quicklyconf.loadConfig(can_stop=False, config_file_path=self.project_root_dir)
        if bzr != self.ui.config_bzr_entry.get_text():
            quicklyconf.project_config["bzrbranch"] = self.ui.config_bzr_entry.get_text()
        if lp != self.ui.config_lp_entry.get_text():
            quicklyconf.project_config["lp_id"] = self.ui.config_lp_entry.get_text()
        if ppa != self.ui.config_ppa_entry.get_text():
            quicklyconf.project_config["ppa"] = self.ui.config_ppa_entry.get_text()
        if target != self.ui.config_target_entry.get_text():
            quicklyconf.project_config["version"] = self.ui.config_target_entry.get_text()
        try: 
            quicklyconf.saveConfig(self.project_root_dir)
            self.ui.statusLabel.set_text(self.project_name+ " " + _("configuration saved"))
        except:
            pass                  
            
        
    def set_project_bts_enabled(self, a):
        self.ui.project_edit_bt.set_sensitive(a)
        self.ui.project_design_bt.set_sensitive(a)
        self.ui.project_save_bt.set_sensitive(a)
        self.ui.project_package_bt.set_sensitive(a)
        self.ui.project_share_bt.set_sensitive(a)
        self.ui.project_release_bt.set_sensitive(a)
        self.ui.editor_design_bt.set_sensitive(a)   
        self.ui.editor_debug_bt.set_sensitive(a)
        self.ui.editor_run_bt.set_sensitive(a)
        self.ui.editor_add_bt.set_sensitive(a)
        self.ui.add_add_bt.set_sensitive(a)
        self.ui.config_bt.set_sensitive(a)
        self.ui.project_run_bt.set_sensitive(a)
        self.ui.project_license_bt.set_sensitive(a) 
   
    def on_add_add_bt_clicked(self, widget):
        index = self.ui.add_type_bt.get_active()
        name = self.ui.add_name_entry.get_text()
        AddType = command_add_types[index]
        args = [AddType, name]
        self.project_run_command("add", args)
        self.refresh_project_view()
        self.ui.statusLabel.set_text(name + " " + (_("Added")))     
            
    def on_edit_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(4)   
        
    def on_edit_undo_bt_clicked(self, widget):
        self.editorBuffer.undo()    
        
    def on_edit_redo_bt_clicked(self, widget):
        self.editorBuffer.redo()    
        
    def on_editor_find_bt_clicked(self, widget):
        text = self.ui.editor_find_entry.get_text()
        start_iter =  self.editorBuffer.get_start_iter()
        found = start_iter.forward_search(text,0, None)
        
        if found:
            match_start,match_end = found #add this line to get match_start and match_end
            self.editorBuffer.select_range(match_start,match_end) 
         
    def on_editor_add_bt_clicked(self, widget):
        self.ui.notebook.set_current_page(5)     
            
    def on_editor_design_bt_clicked(self, widget):
        self.project_run_command("design")
    
    def on_editor_debug_bt_clicked(self, widget):
        self.project_run_command("debug")
        
    def on_editor_run_bt_clicked(self, widget):
        self.project_run_command("run")               
        
    def editor_load_file(self, filePath):
        if filePath == None: return
        try:
            content = open(filePath).read()
        except:
            print "Cannot open file"  
        self.editorBuffer.begin_not_undoable_action () 
        lang = self.editLangMan.guess_language(filePath, None)
        self.editorBuffer.set_language(lang)      
        self.editorBuffer.set_text(str(content)) 
        self.editor_current_file = filePath
        self.editorBuffer.end_not_undoable_action () 
                   
    def on_edit_open_bt_clicked(self, widget):
        dialog = Gtk.FileChooserDialog(title=_("Open..."),action=Gtk.FileChooserAction.OPEN,
                        buttons=(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OPEN,Gtk.ResponseType.ACCEPT))
        Filefilter = Gtk.FileFilter()
        Filefilter.set_name(_("All files"))
        Filefilter.add_pattern("*")
        dialog.add_filter(Filefilter)
          
        Filefilter = Gtk.FileFilter()
        Filefilter.set_name(_("Python files"))
        Filefilter.add_mime_type("text/x-python")
        dialog.add_filter(Filefilter)
          
        response = dialog.run()
        fileName = None
        if response == Gtk.ResponseType.ACCEPT:
            fileName = dialog.get_filename()
        elif response == Gtk.ResponseType.CANCEL:
            dialog.destroy()
            return
        dialog.destroy()
        self.editor_load_file(fileName)
        
    def on_edit_save_bt_clicked(self, widget):
        if len(self.editor_current_file) == 0:
            self.on_edit_save_as_bt_clicked(self)
            return
        else:
            f = open(self.editor_current_file, 'w')
            start, end = self.editorBuffer.get_bounds()
            content = self.editorBuffer.get_text(start, end, True)
            f.write(content)
            self.ui.statusLabel.set_text(_("Saved"))
            f.close()
            self.editor_load_file(self.editor_current_file)
            
    def on_edit_save_as_bt_clicked(self, widget):
        dialog = Gtk.FileChooserDialog(title=_("Save as..."),action=Gtk.FileChooserAction.SAVE,
                        buttons=(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_SAVE,Gtk.ResponseType.ACCEPT))
        
        
        response = dialog.run()
        if response == Gtk.ResponseType.ACCEPT:
            fileName = dialog.get_filename()
        elif response == Gtk.ResponseType.CANCEL:
            dialog.destroy()
            return
        dialog.destroy()
        
        f = open(fileName, 'w')
        start, end = self.editorBuffer.get_bounds()
        content = self.editorBuffer.get_text(start, end, True)
        f.write(content)
        self.ui.statusLabel.set_text("Saved")
        f.close()
        
        if len(self.editor_current_file) == 0:
            self.editor_current_file = fileName
        
    def on_edit_new_bt_clicked(self, widget):
        self.editorBuffer.set_text("")
        self.editor_current_file = ""
       
    def save_state(self):
        # Write window settings
        size = self.get_size()
  
        config = ConfigParser.RawConfigParser()
        config.add_section('Window')
        config.set('Window', 'width', size[0])
        config.set('Window', 'height', size[1])
        
        if not os.path.exists(CONFIG_PATH): os.makedirs(CONFIG_PATH)
        
        path = str(CONFIG_PATH + "window.cache")
        
        with open(path, 'wb') as configfile:
            config.write(configfile)
        # Write Projects DB
        path = str(CONFIG_PATH + "projects.db")
         
        with open(path, 'wb') as projectsfile: 
            pickle.dump(self.projects_list, projectsfile)
              
    def restore_state(self):
        if not os.path.exists(CONFIG_PATH): return
        
        config = ConfigParser.RawConfigParser()
        try:
            config.read(str(CONFIG_PATH + "window.cache"))
        except:
            return    
        try:
            width = config.getint('Window', 'width')
            height = config.getint('Window', 'height')
        except:
            return    
        self.resize(width, height)
        
        self.load_projects_list()
            
    def load_projects_list(self):
        path = str(CONFIG_PATH + "projects.db")
        
        if os.path.exists(path):
            f = open(path, "rb")
            self.projects_list = pickle.load(f)
            f.close()
            return
        else:        
            self.projects_list = []        
       
    def is_file_type(self, fileName, Filetype):
        if fileName == None:
            return
            
        uiTypes = (".ui", ".glade")    
        imageTypes = (".png", ".jpg", ".svg", ".jpeg", ".bmp", ".gif", ".ico")    
            
        if Filetype == "ui":
            if fileName.endswith(uiTypes):
                return True
            else:
                return False           
        elif Filetype == "image":
            if fileName.endswith(imageTypes):
                return True
            else:
                return False
        elif Filetype == "trash":
            if fileName.endswith("~"):
                return True
            else:
                return False                     
        elif Filetype == "pytrash":
            if fileName.endswith(".pyc"):
                return True
            else:
                return False        

